pub trait iinstruction {
    fn toInstruction(&self) -> String {
        String::from("not impliment...")
    }
}

pub trait ichannel {
    /// 发送消息的抽象方法
    ///     参数：
    ///         instruction: 要发送的指令
    ///     返回结果：
    ///         (bool, String) -> (发送状态, 消息)
    fn send(instruction: &iinstruction) -> (bool, String);
}

/// 巡检结果：OCR 智能识别
#[derive(Debug)]
pub struct Instruction_OCR {
    /*
    R##DA={0}##DN=##CID={1}##TY=2##TID={2}##SC={3}##DR={4}##PID={5}##V={6}##BC={7}##AID={8}##T=0##E=##TM={9:yyyyMMddHHmmss}^^
    {0}: 采集器编号
    {1}: 巡检目标设备Id
    {2}: 巡检任务Id
    {3}: 工位号
    {4}: 方位（左/右）
    {5}: 属性Id
    {6}: 值
    {7}: 批次号
    {8}: 地点Id
    {9}: 采集时间
    */
    pub agentbm: String,
    pub addr_id: String,
    pub plan_id: i32,
    pub station_code: String,
    pub derection: bool,
    pub device_id: String,
    pub property_id: String,
    pub value: String,
    pub batch: String,
    pub time: String,
}

/// 巡检结果：红外扫描
#[derive(Debug)]
pub struct Instruction_Temp {
    /*
    R##DA={0}##DN=##CID={1}##TY=3##TID={2}##SC={3}##DR={4}##PID={5}##V={6}##SV={7}##LV={8}##IX={9}##IY={10}##BC={11}##T=0##E=##TM={12:yyyyMMddHHmmss}^^
    {0} : 采集器编号
    {1} : 巡检目标设备Id
    {2} : 巡检任务Id
    {3} : 工位号
    {4} : 方位（左/右）
    {5} : 属性Id
    {6} : 值
    {7} : 最小值
    {8} : 最大值
    {9} : x轴偏差
    {10}: y轴偏差
    {11}: 批次号
    {12}: 采集时间
    */
    pub agentbm: String,
    pub addr_id: String,
    pub plan_id: i32,
    pub station_code: String,
    pub derection: bool,
    pub device_id: String,
    pub property_id: String,
    pub value: String,
    pub value_min: String,
    pub value_max: String,
    pub value_ix: String,
    pub value_iy: String,
    pub batch: String,
    pub time: String,
}

impl iinstruction for Instruction_OCR {
    fn toInstruction(&self) -> String {
        let mut derection: i32 = 0;
        if self.derection {
            derection = 1;
        } else {
            derection = 0;
        }

        format!("R##DA={}##DN={}##CID={}##TY=2##TID={}##SC={}##DR={}##PID={}##V={}##BC={}##AID={}##T=0##E=##TM={}^^"
                /* DA */, &self.agentbm
                /* DN */, ""
                /* CID */, &self.device_id
                /* TID */, &self.plan_id
                /* SC */, &self.station_code
                /* DR */, derection
                /* PID */, &self.property_id
                /* V */, &self.value
                /* BC */, &self.batch
                /* AID */, &self.addr_id
                /* TM */, &self.time)
    }
}

impl iinstruction for Instruction_Temp {
    /*
    fn toInstruction(&self) -> String {
        let derection = match (&self.derection == true) {
            true => 1,
            _ => 0
        };

        format!("R##DA={0}##DN=##CID={1}##TY=3##TID={2}##SC={3}##DR={4}##PID={5}##V={6}##SV={7}##LV={8}##IX={9}##IY={10}##BC={11}##T=0##E=##TM={12:yyyyMMddHHmmss}^^"
                /* DA */, &self.agentbm
                /* DN */, ""
                /* CID */, &self.device_id
                /* TID */, &self.plan_id
                /* SC */, &self.station_code
                /* DR */, derection
                /* PID */, &self.property_id
                /* V */, &self.value
                /* SV */, &self.value_min
                /* LV */, &self.value_max
                /* IX */, &self.ix
                /* IY */, &self.iy
                /* BC */, &self.batch
                /* AID */, &self.addr_id
                /* TM */, &self.time);
    }
    */
}


#[cfg(test)]
mod unittest{
    use super::*;

    #[test]
    fn test_ocr(){
        let ocr = Instruction_OCR{
            agentbm: String::from("99"),
            addr_id: String::from("F-13-01"),
            plan_id: 20,
            station_code: String::from("SC-1301-04"),
            derection: false,
            device_id: String::from("SEV-HP-P20-03"),
            property_id: String::from("HUM"),
            value: String::from("35.8"),
            batch: String::from("20170718123000"),
            time: String::from("20170718123218"),
        };
        println!("{:?}",ocr);
        println!("Target Instruction: {}",ocr.toInstruction());
    }
}