extern crate ConExpression;
extern crate rand;

mod test;
mod network;
mod hashmap;
mod math;
mod icontract;



use std::io;
use std::io::Read;
use std::cmp::Ordering;
use std::env;
use std::fs::File;
use std::process;
use std::process::Command;
use std::thread;
use std::time::Duration;

use rand::Rng;
use icontract::iinstruction;

#[derive(Debug)]
struct Rectangle {
    width: u32,
    height: u32,
}
impl Rectangle {
    fn area(&mut self) -> u32 {
        self.width = self.width + 10;
        self.width * self.height
    }
}

#[derive(Debug)]
enum IpKind {
    V4,
    V6,
}

#[derive(Debug)]
struct IpAddr
{
    kind: IpKind,
    address: String,
}

#[derive(Debug)]
enum IpAddr2
{
    V4(String),
    V6
}

#[derive(Debug)]
enum IpAddr3
{
    V4(u8, u8, u8, u8),
    V6
}

#[derive(Debug)]
struct Config {
    query: String,
    filename: String
}

impl Config {
    fn new(args: &[String]) -> Result<Config, &str> {
        if args.len() < 3 {
            return Err("not enought arguments");
        }

        Ok(Config {
            query: args[1].clone(),
            filename: args[2].clone(),
        })
    }
}

fn parse_config1(args: &[String]) -> (&str, &str) {
    if args.len() < 3 {
        return ("NULL", "NULL");
    }

    let query = &args[1];
    let filename = &args[2];

    (query, filename)
}
fn parse_config2(args: &[String]) -> Config {
    if args.len() < 3 {
        return Config { query: String::from("NONE"), filename: String::from("NONE") };
    }

    let query = args[1].clone();
    let filename = args[2].clone();

    Config { query, filename }
}
fn parse_config3(args:&[String])->Result<Config, &str> {
    if args.len() < 3 {
        return Err("no enought arguments");
    }

    Ok(Config {
        query: args[0].clone(),
        filename: args[0].clone(),
    })
}
fn read_content_from_file() -> Result<String, io::Error> {
    //let filename="./hello.txt";
    let filename="F:/Work/Rust/ConExpression/target/debug/hello.txt";
    let mut s = String::new();
    let f = File::open(filename)?;

    File::open(filename)?.read_to_string(&mut s)?;

    Ok(s)
}

struct Cacher<T>
    where T:Fn(i32)->i32 {
    calculation: T,
    value: Option<i32>,
}
impl<T> Cacher<T>
    where T:Fn(i32)->i32 {
    fn new(calculation: T) -> Cacher<T> {
        Cacher {
            calculation: calculation,
            value: None,
        }
    }

    fn value(&mut self, arg: i32) -> i32 {
        match self.value {
            Some(v) => v,
            None => {
                let v = (self.calculation)(arg);
                self.value = Some(v);
                v
            }
        }
    }
}


#[derive(Debug)]
struct Counter{
    count:u32
}
impl Counter {
    fn new() -> Counter {
        Counter {
            count: 0
        }
    }
}
impl Iterator for Counter {
    type Item = u32;

    fn next(&mut self) -> Option<Self::Item> {
        self.count += 1;
        //return Some(self.count);

        if self.count < 6 {
            Some(self.count)
        } else {
            None
        }
    }
}

fn main() {
    println!();
    println!("=================================================================================================");
    println!();

    println!();
    println!("-- test Iterator.filter ----------------------------------------------------");
    let str="
Tom: Hello, where are you?
Lily: I am here.
Tom: Are you Ok?
Lily: Yes, I am fine!";
    let str_vec:Vec<&str>=str.lines().filter(|x|x.to_lowercase().contains("are")).collect();
    println!("{:#?} - {:#?}",str, str_vec);

    println!();
    println!("-- test Iterator ----------------------------------------------------");
    let mut counter = Counter::new();
    println!("count 1: {:?}", counter.next());
    println!("count 2: {:?}", counter.next());
    println!("count 3: {:?}", counter.next());
    println!("count 4: {:?}", counter.next());
    println!("count 5: {:?}", counter.next());
    println!("count 6: {:?}", counter.next());
    println!("count 7: {:?}", counter.next());
    println!("counter: {:?}", counter);

    println!();
    let counter1: Vec<u32> = Counter::new().collect();
    println!("counter1: {:?}", counter1);
    let counter2: Vec<u32> = Counter::new().skip(2).collect();
    println!("counter2: {:?}", counter2);
    let counter_zip = Counter::new().zip(Counter::new().skip(2));
    println!("counter_zip: {:?}", counter_zip);
    let counter_map = counter_zip.map(|(a, b)| a * b);
    println!("counter_map: {:?}", counter_map);
    //let counter_map_coll: Vec<u32> = counter_map.collect();
    //println!("counter_map_coll: {:?}", counter_map_coll);
    let counter_filter = counter_map.filter(|x| x % 3 == 0);
    println!("counter_filter: {:?}", counter_filter);
    //let counter_filter_coll :Vec<u32>=counter_filter.collect();
    //println!("counter_filter_coll: {:?}", counter_filter_coll);
    let counter_sum: u32 = counter_filter.sum();
    println!("counter_sum: {:?}", counter_sum);

    println!();
    println!("-- test Closure ----------------------------------------------------");
    let intensity = 21;
    let random_number = 7;
    test_closure(intensity, random_number);

    ConExpression::run();

    /*
    println!("=============================");
    println!("{:?}", read_content_from_file().unwrap());
    println!("=============================");
*/

    match read_content_from_file() {
        Ok(str) => println!("OK: {:?}", str),
        Err(e) => println!("Err: {:?}", e)
    }
    let args: Vec<String> = env::args().collect();
    println!("★★★ arguments: {:?}", args);

    let config1 = parse_config1(&args);
    println!("★★★ funtion 1 to parse arguments: {:?}", config1);

    let config2 = parse_config2(&args);
    println!("★★★ funtion 2 to parse arguments: {:?}", config2);

    let config3 = parse_config3(&args);
    println!("★★★ funtion 3 to parse arguments: {:?}", config3);

    let config4 = Config::new(&args);
    println!("★★★ funtion 4 to parse arguments: {:?}", config4);

    let config4 = config4.unwrap_or_else(|err| {
        println!("parsing arguments err: {}", err);
        process::exit(0);
    });

    let mut file = File::open(config4.filename).expect("file not Found");
    let mut content: String = String::from("");
    file.read_to_string(&mut content);
    println!("\n  Content:\n{}", content);

    let robot_agent_bm: &'static str = "R1";
    let robot_device_id: &'static str = "99";

    return;

    println!();
    println!("-- test lifetime ----------------------------------------------------");
    {
        let x = "abcd";
        let y = "xyz";
        let z = longest(x, y);
        println!("maxLen(\"{}\", \"{}\") = \"{}\"", x, y, z);
    }

    println!();
    println!("-- test instruction: Temp ----------------------------------------------------");
    let temp = icontract::Instruction_Temp {
        agentbm: String::from("99"),
        addr_id: String::from("F-13-01"),
        plan_id: 20,
        station_code: String::from("SC-1301-04"),
        derection: false,
        device_id: String::from("SEV-HP-P20-03"),
        property_id: String::from("HUM"),
        value: String::from("=[{x:0,y:0,v:21.61},{x:0,y:4,v:20.62},{x:0,y:8,v:25.70},{x:0,y:12,v:20.73},{x:4,y:0,v:23.55},{x:4,y:4,v:25.57}]"),
        value_min: String::from("{x:0,y:4,v:20.62}"),
        value_max: String::from("{x:0,y:8,v:25.70}"),
        value_ix: String::from("4"),
        value_iy: String::from("4"),
        batch: String::from("20170718123000"),
        time: String::from("20170718123218"),
    };
    println!("{:?}", temp);
    println!("Target Instruction: {}", temp.toInstruction());

    println!();
    println!("-- test instruction: OCR ----------------------------------------------------");
    let ocr = icontract::Instruction_OCR {
        agentbm: String::from("99"),
        addr_id: String::from("F-13-01"),
        plan_id: 20,
        station_code: String::from("SC-1301-04"),
        derection: false,
        device_id: String::from("SEV-HP-P20-03"),
        property_id: String::from("HUM"),
        value: String::from("35.8"),
        batch: String::from("20170718123000"),
        time: String::from("20170718123218"),
    };
    println!("{:?}", ocr);
    println!("Target Instruction: {}", ocr.toInstruction());

    println!();
    println!("-- test funtion ----------------------------------------------------");
    let num_list = vec!(10, 3, 29, 40, 2);
    println!("the largest NUMBER is: {}", math::get_largest(&num_list));

    let num_list = vec!('a', 'i', 'g', 'b', 'z');
    println!("the largest CHAR is: {}", math::get_largest(&num_list));

    let num_list = vec!("Apple", "Orange", "Peach", "Banana");
    println!("the largest STRING is: {}", math::get_largest(&num_list));

    use std::net::IpAddr;
    let home = "127.0.0.1".parse::<IpAddr>().unwrap();
    //let home = "127.300.0.1".parse::<IpAddr>().unwrap();

    //panic!("###################################\npanic here...\n###################################");

    println!();
    println!("-- test Map ----------------------------------------------------");
    hashmap::test();

    println!();
    println!("-- test Mod ----------------------------------------------------");
    network::server::http_server::connect();
    network::server::connect();
    network::connect();
    test::connect();

    println!();
    println!("-- test Enum ----------------------------------------------------");
    test_enum();

    println!();
    println!("-- test Struct =====================================================");
    let mut rect = Rectangle { width: 30, height: 50 };
    test_rectangle(&mut rect);
    println!("main(): {:?}", rect);

    println!();
    println!("-- test For ----------------------------------------------------");
    test_for();

    println!();
    println!("-- test xxx ----------------------------------------------------");
    let x = 6;
    println!("outer x: {}", x);
    let y = {
        println!("inner x before re-defined: {}", x);
        let x = 3;
        println!("inner x after re-defined: {}", x);
        x + 1
    };
    println!("outer x: {}", x);
    println!("y: {}", y);

    /**/
    guess_number();

    let output = Command::new("rustc")
        .arg("--version")
        .output().unwrap_or_else(|e| {
        panic!("failed to execute process: {}", e)
    });

    if output.status.success() {
        let s = String::from_utf8_lossy(&output.stdout);
        print!("rustc succeeded and stdout was:\n{}", s);
    } else {
        let s = String::from_utf8_lossy(&output.stderr);

        print!("rustc failed and stderr was:\n{}", s);
    }

    println!();
    println!();
    println!();
    println!("\t\"Enter\" to Exit...");
    let mut exit = String::new();
    io::stdin().read_line(&mut exit).expect("err.");


    println!();
    println!("=================================================================================================");
}

fn longest<'a>(x:&'a str,y:&'a str)-> &'a str {
    if x.len() > y.len() {
        x
    } else {
        y
    }
}

fn test_closure(intensity:i32,random_number:i32) {
    /*
    if intensity < 25 {
        println!("Today, do {} pushups!",
                 simulated_expensive_calculation(intensity));

        println!("Next, do {} situps!",
                 simulated_expensive_calculation(intensity));
    } else {
        if random_number == 3 {
            println!("Take a break day! Remember to stay hydrated.");
        } else {
            println!("Today, run for {} minutes!",
                     simulated_expensive_calculation(intensity));
        }
    }
    */

    let mut expensive_result = Cacher::new(|i| {
        println!("calculating slowly...");
        thread::sleep(Duration::from_secs(2));
        i
    });

    if intensity < 25 {
        println!("Today, do {} pushups!",
                 expensive_result.value(intensity));

        println!("Next, do {} situps!",
                 expensive_result.value(intensity));
    } else {
        if random_number == 3 {
            println!("Take a break day! Remember to stay hydrated.");
        } else {
            println!("Today, run for {} minutes!",
                     expensive_result.value(intensity));
        }
    }

    let c = |x: i32| x + 1;
    let b = c(3);
    println!("{:?}", b);
}

fn test_enum() {
    let home1 = IpAddr { kind: IpKind::V4, address: String::from("127.0.0.1") };
    println!("IP: {:#?}", home1);
    println!("\tKind: {:?}", home1.kind);
    println!("\tAddress: {:?}", home1.address);
    let home2 = IpAddr2::V4(String::from("127.0.0.1"));
    println!("IP: {:#?}", home2);

    let home3 = IpAddr3::V4(12, 23, 34, 45);
    println!("IP: {:#?}", home3);
}

fn test_rectangle( rect: &mut Rectangle) {
    println!("\ttest_rectangle() rect is {:?}", rect);
    rect.height = rect.height + 10;
    println!("\ttest_rectangle() the height is changed to {}", rect.height);
    println!("\ttest_rectangle() Step 1: area is {}, width = {}, height = {}", rect.area(), rect.width, rect.height);
    println!("\ttest_rectangle() Step 2: area is {}, width = {}, height = {}", rect.area(), rect.width, rect.height);
}

fn test_for() {
    // for 1
    let numbers = [10, 20, 30, 40];
    for ele in numbers.iter() {
        println!("for1 - the value is: {}", ele);
    }

    // for 2
    for ele in (1..5).rev() {
        println!("for2 - the value is: {}", ele);
    }
}

fn guess_number() {
    println!();
    let secret_number = rand::thread_rng().gen_range(1, 101);
    loop {
        println!("please input your guess number:");
        let mut guess_number = String::new();
        io::stdin().read_line(&mut guess_number).expect("failed to read number.");
        //let int_number: i32= guess_number.trim().parse().expect("not a number.");
        let guess_number: u32 = match guess_number.trim().parse(){
            Ok(num)=>num,
            Err(_)=> {
                println!("expect a number!");
                continue;
            }
        };

        println!("You guessed: {}", guess_number);
        match guess_number.cmp(&secret_number) {
            Ordering::Less => println!("Too small!"),
            Ordering::Greater => println!("Too big!"),
            Ordering::Equal => {
                println!("You win!");
                break;
            }
        };
    };
}

fn simulated_expensive_calculation(intensity:i32)->i32 {
    println!("calculating slowly...");
    thread::sleep(Duration::from_secs(2));
    intensity
}